<?php

namespace Drupal\Tests\commerce_registration\Kernel;

use Drupal\commerce_registration\Access\ManageCommerceRegistrationsAccessCheck;
use Drupal\Core\Routing\RouteMatch;
use Drupal\Tests\commerce_registration\Traits\ProductCreationTrait;
use Drupal\Tests\commerce_registration\Traits\ProductVariationCreationTrait;
use Symfony\Component\Routing\Route;

/**
 * Tests the manage commerce registrations access checker.
 *
 * @coversDefaultClass \Drupal\commerce_registration\Access\ManageCommerceRegistrationsAccessCheck
 *
 * @group commerce_registration
 */
class ManageCommerceRegistrationsAccessCheckTest extends CommerceRegistrationKernelTestBase {

  use ProductCreationTrait;
  use ProductVariationCreationTrait;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $admin_user = $this->createUser();
    $this->setCurrentUser($admin_user);
  }

  /**
   * @covers ::access
   */
  public function testManageCommerceRegistrationsAccessCheck() {
    $registration_manager = $this->container->get('registration.manager');
    $access_checker = new ManageCommerceRegistrationsAccessCheck($this->entityTypeManager, $registration_manager);

    $product = $this->createAndSaveProduct();
    $variation = $this->createAndSaveVariation($product);

    $route = new Route('/product/{commerce_product}/registrations');
    $route
      ->addDefaults([
        '_controller' => '\Drupal\commerce_registration\Controller\CommerceRegistrationController::manageRegistrations',
        '_title_callback' => '\Drupal\commerce_registration\Controller\CommerceRegistrationController::manageRegistrationsTitle',
      ])
      ->addRequirements([
        '_manage_commerce_registrations_access_check' => 'TRUE',
      ])
      ->setOption('_admin_route', TRUE)
      ->setOption('parameters', [
        'commerce_product' => ['type' => 'entity:commerce_product'],
      ]);
    $route_name = 'entity.commerce_product.commerce_registration.manage_registrations';
    $route_match = new RouteMatch($route_name, $route, [
      'commerce_product' => $product,
    ]);

    // Administer registration permission.
    $account = $this->createUser([], ['administer registration']);
    $access_result = $access_checker->access($account, $route_match);
    $this->assertTrue($access_result->isAllowed());

    // Administer "type" registration permission.
    $account = $this->createUser([], [
      'administer conference registration',
      'view any registration',
    ]);
    $access_result = $access_checker->access($account, $route_match);
    $this->assertTrue($access_result->isAllowed());

    // Insufficient permission.
    $account = $this->createUser([], [
      'administer conference registration',
    ]);
    $access_result = $access_checker->access($account, $route_match);
    $this->assertFalse($access_result->isAllowed());

    // Administer "own" registration permission.
    $account = $this->createUser([], [
      'administer own conference registration',
      'administer commerce_product',
      'view any registration',
    ]);
    $access_result = $access_checker->access($account, $route_match);
    $this->assertTrue($access_result->isAllowed());

    // Insufficient permission.
    $account = $this->createUser([], [
      'administer own conference registration',
      'view any registration',
    ]);
    $access_result = $access_checker->access($account, $route_match);
    $this->assertFalse($access_result->isAllowed());

    // Insufficient permission.
    $account = $this->createUser([], [
      'administer commerce_product',
      'view any registration',
    ]);
    $access_result = $access_checker->access($account, $route_match);
    $this->assertFalse($access_result->isAllowed());

    // Insufficient permission.
    $account = $this->createUser([], ['access registration overview']);
    $access_result = $access_checker->access($account, $route_match);
    $this->assertFalse($access_result->isAllowed());
  }

}
